### TO SETUP ###
FINAL_MODELS_DIR = '/auto/plzen4-ntis/projects/BioML/tomas_lukas/final_models/'
RESULTS_DIR = '/auto/plzen4-ntis/projects/BioML/tomas_lukas/results/'
DATA_DIR = '/storage/plzen4-ntis/projects/BioML/datasets/proteoconv/'
VAL_DIR = '/auto/plzen4-ntis/projects/BioML/tomas_lukas/data/'
STATS_DIR = '/auto/plzen4-ntis/projects/BioML/tomas_lukas/statistics'

### CONCATENATED ###
REFPROT_TSV = VAL_DIR + 'refprot_random_taxfree.tsv'
TAX_TSV = VAL_DIR + 'tax_ds_1000_prots_from_each_order.tsv'

### OTHER ###
TOT_TRAIN_PROT = 47082110
